#ifndef CIS_H
#define CIS_H

#include "MyTools.h"

#include "seed.h"
#include "iterativescan.h"

#include <queue>

/**
 * @class IterativeDensity
 * @brief Connected Iterative Scan
 * 
 * Implementation of the Connected Iterative Scan algorithm. Many functions are overwritten here, as the original authors 
 * 	used the boost network library, which requires special treatment.
 * 
 * @author Robert Escriva
 */
template <typename R>
class IterativeDensity
{
  public:
    /**
     * Constructor
     * 
     * 	Simply initializes some variables.
     * 
     * @param args lambda value and density function to use ("-l 0.2 -d c")
     */
  IterativeDensity(double l, density<bidirected_t>* d, bool t){
      lambda = l;
      dens = d;
                  
      this->translate = t;
      this->fileout = &cout;
    };
    
    
    /**
     * Destructor
     *
     *	Empty 
     */
    ~IterativeDensity(){};
    
    /**
     * Implementation
     */
    void calculate();
    
    /**
     * Read a network from a file into a boost network structure
     * 
     * @param fin Stream to read from
     * @param delimiters Delimiters in network file
     * @return true
     */
    bool ReadGraph(ifstream* fin, string delim){
      delimiters = delim;      
      read_from_stream<bidirected_t>(graph, map, (this->idmap), *fin, 1, 1, 0, delimiters);
      return true;
    };
    
    /**
     * Print communities using boost structures for the network
     * 
     * @param filename Name of file to print communities to.
     * @return true
     */
    bool PrintCommunities(string filename){
      ofstream fout(filename.c_str());
      for (int i = 0; i < comms.size(); i++){
	print_set<bidirected_t>(graph, comms[i].vertices(), fout, comms[i].dens());
      }
      return true;
    };
    
    /**
     * Print communities using boost structures and the original vertex names.
     * 
     * @param filename Name of file to print communities to.
     * @return true
     */
    bool PrintTranslatedCommunities(string filename){
      ofstream fout(filename.c_str());
      for (int i = 0; i < comms.size(); i++){
	print_set_trans<bidirected_t>(graph, comms[i].vertices(), fout, comms[i].dens(), this->idmap, delimiters[0]);
      }
      return true;
    };
    
    /**
     * Translates communities from boost structure ID to original ID, provided the original ID is also an integer.
     * 
     * @return true
     */    
    template <typename graph_t>
    bool TranslateCommunities(){
      std::map <long unsigned int, string> rID;
      std::map <long unsigned int, string>::iterator it_r;
      std::map <string, long unsigned int>::iterator it_id;
      set <lui>::iterator it_sl; 
      
      for (it_id = (this->idmap).begin(); it_id != (this->idmap).end(); it_id++){
	rID.insert(make_pair<lui, string>(it_id->second, it_id->first));
      }
      
      for (int i = 0; i < comms.size(); i++){
	set<typename graph_t::vertex> v = comms[i].vertices();
	if (v.size() > 2)
	{
	  set <lui> clust;
	  typename std::set<typename graph_t::vertex>::const_iterator i;
	  for (i = v.begin(); i != v.end(); ++i)
	  {
	      typename graph_t::vertex_name_t u;
	      u = boost::get(boost::vertex_name, graph, *i);
	      clust.insert(boost::lexical_cast<lui>((rID.find(boost::lexical_cast<lui>(u)))->second));
	  }
	  
	  (this->communities).push_back(clust);
	}
      }
      return true;
    };

    /**
     * Set the output file for results
     *
     *@param filename Results file
     */
    void setOutput(string filename){
      fileout = new ofstream(filename.c_str());
    };

    std::map<string, lui> idmap;

  private:
    double lambda; /**< @see Constructor*/
    vector <seed> comms; /**< Holds the detected communities*/
    bidirected_t::graph graph; /**< Boost network structure*/
    bidirected_t::map map; /**< Boost network structure */    
    bool translate; /**< Whether or not to convert node IDs back to original names*/
    ostream* fileout; /**< Results file*/
    string delimiters; /**< Network file delimiter characters*/

    density<bidirected_t>* dens; /**< Density calculation class*/

    /** 
    * @class future_actions
    *  @brief Used during CIS algorithm to decide what to do if a seed becomes disconnected.
    */
    class future_actions
    {
	public:
	    typedef std::pair<const std::vector<seed>*, double> results;

	    /**
	    * Destructor
	    */
	    virtual ~future_actions(){};
	    
	    /**
	    * Constructor
	    */
	    virtual void operator ()(const results, std::queue<seed>* q,
		    std::vector<seed>* ret) =0;
    };

    /**
    * @class take_best
    * @brief If a seed becomes disconnected, take the component with the highest density to continue
    * 
    */
    class take_best: public IterativeDensity<R>::future_actions
    {
	public:
	  /**
	   * Constructor
	   * 	Simple parameter setting
	   * 
	   * @param threshold Keep at 0
	   */	  
	    take_best(double threshold=0):thresh_(threshold){};
	    
	    /**
	     * Implementation function
	     * 
	     * @param r Results from expansion
	     * @param q [out] the best seed 
	     */
	    void operator ()(const typename IterativeDensity<R>::future_actions::results r, std::queue<seed>* q,
		    std::vector<seed>* ret){
	    /*Makes sure there is a graph to work with*/
		if ((*r.first).size() < 1)
		{
		    throw "ret too short";
		}
	    /*Initialize*/
		seed best = (*r.first)[0];

	    /*Find the disjoint subgraph from results.first with the highest connectivity.*/
		for (unsigned int i = 0; i < r.first->size(); ++i)
		{
		    if (best.dens() < (*r.first)[i].dens())
		    {
			best = (*r.first)[i];
		    }else if (best.dens() == (*r.first)[i].dens() && best.vertices().size() < (*r.first)[i].vertices().size())
		    {
		      
		    }
		}

	    /*Depending on whether or not the set's connectiveness is good enough, sort it*/
	    /*NOTE: should be best.dens() ?*/
		if (r.second <= thresh_)
		{
		    ret->push_back(best);
		}
		else
		{
		    q->push(best);
		}
	    };

		private:
		    double thresh_; /**< @see Constructor*/
	    };
  
  /**
     * Main implementation loop
     * 
     * @param S Seed cluster to expand
     * @param act Action to take if seed becomes disconnected
     * @param graph Boost network
     * 
     * @return The set of vertices that results from expanding the given seed in the given network
     */
    std::vector<seed>* cluster(const seed& S, future_actions* act, bidirected_t::graph* graph)
    {
      //Start with seed
      std::vector<seed>* ret = new std::vector<seed>();
      std::queue<seed> to_consider;
      to_consider.push(S);

      //Keep expanding seed until convergence
      while (!to_consider.empty())
      {
	  seed s = to_consider.front();
	  to_consider.pop();
	  
	  std::pair<std::vector<seed>*, double> results;
	  results = expand_seed(s);

	  (*act)(results, &to_consider, ret);
	  
	  delete results.first;
      }

      return ret;
    }

};

#endif
