/**
 * @file DriveCIS.cc
 * @brief Standalone Implementation of CIS Algorithm
 * 
 * Program to run CIS clustering on a network.
 */

#include "MyTools.h"
#include "CIS.h"

#include <boost/lexical_cast.hpp>

/**
 * @param -l lambda value to use for density calculations
 * @param -d ('c' or 'n') type of density to use. 'c' for complex - for use in positive weighted network. 'n' for negative - for use in networks with negative edges
 * @param -i network input file
 * @param -o results file
 * @param -dl delimiter used in network file
 * @param -tr Convert node IDs back to original names in results file (translate)
 */
int main(int argc, char** argv){
  
  /* Retrieve Parameters*/
  map <string, vector <string> > params = getParameters(argc, argv);
  map <string, vector <string> >::iterator it_p;

  double lambda = 0;
  density<bidirected_t>* dens;
  bool translate = false;
  string delimiters = " \t";
  string networkFile, outputFile;

  if ((it_p = params.find("l")) != params.end()){
    lambda = boost::lexical_cast<double>(it_p->second[0]);
  } else {
    cerr << "Using default lambda value of 0" << endl;
  }

  if ((it_p = params.find("d")) != params.end()){
    switch ((it_p->second[0])[0])
    {
    case 'c':
      dens = new complex_density<bidirected_t>(lambda);
      break;
    case 'n':
      dens = new negative_density<bidirected_t>(lambda);
      break;
    default:
      dens = new complex_density<bidirected_t>(lambda);
      cout << "Invalid entry for density type (should be n or c) - Using complex density (c)" << endl;
      break;
    }
  } else {
    dens = new complex_density<bidirected_t>(lambda);
    cout << "Using default of complex density" << endl;
  }

  if ((it_p = params.find("tr")) != params.end()){
    cout << "Translating back to original node names." << endl;
    translate = true;
  } else {
    cout << "Printing ID file for later translation." << endl;
    translate = false;
  }

  if ((it_p = params.find("dl")) != params.end()){
    delimiters = it_p->second[0];
    cout << "Using delimiters " << delimiters << endl;
  } else {
    cout << "Using default delimiters of [space] and [tab]" << endl;
  }

  if ((it_p = params.find("i")) != params.end()){
    networkFile = it_p->second[0];
  } else {
    cerr << "No input file specified." << endl;
    exit(5);
  }

  if ((it_p = params.find("o")) != params.end()){
    outputFile = it_p->second[0];
  } else {
    outputFile = "CIS_results.dat";
  }

  cout << "Printing results to " << outputFile << endl;

  /*Perform Clustering*/
  IterativeDensity<double>* cis = new IterativeDensity<double>(lambda, dens, translate);
  
  ifstream fin;
  if (!openFile(&fin, networkFile.c_str()))
    {
      cerr << "Could not open the network file, please double check if the file exists." << endl;
      exit(2);
    }
  cis->setOutput(outputFile);

  cis->ReadGraph(&fin, delimiters);
  cis->calculate();
  if (!translate)
    WriteMap<string, lui>(&(cis->idmap), (outputFile + "_ID").c_str());

  fin.close();
  delete cis;
}
