/**
 * @file MyTools.cc 
 * @brief Implementation of helper functions
 */

#include "MyTools.h"

/**
 * @fn bool openFile(ifstream* fin, string filename)
 *
 * Open an ifstream with some basic checks
 *
 * @param fin ifstream to use to open file
 * @param filename file to open
 * @return if open successful
 */
bool openFile(ifstream* fin, string filename)
{
  fin->open(filename.c_str());
  if (!(fin->is_open()))
  {
    return false;
  }
  
  return true;
}

/**
 * @fn const std::string trim (const std::string& pString, const std::string& pWhitespace)
 * Remove beginning and trailing "whitespace" from the string
 *
 * @param pString String to trim
 * @param pWhitespace Characters to trim at the ends of the given strings.
 * @return string with whitespace characters stripped from beginning and end
 */
const std::string trim(const std::string& pString,
                       const std::string& pWhitespace)
{
  if ((pString).compare("") == 0)
    return pString;
  
    const size_t beginStr = pString.find_first_not_of(pWhitespace);
    if (pString[beginStr] == 0)
      return "";
    
    if (beginStr == std::string::npos)
    {
        // no content
        return "";
    }

    const size_t endStr = pString.find_last_not_of(pWhitespace);
    const size_t range = endStr - beginStr + 1;

    return pString.substr(beginStr, range);
}

/**
 * @fn bool fline_tr(ifstream* fin, vector <string> fields, string delim)

 * Retrieve a line from fin using delimiters in delim, and trim all strings retrieved.
 *
 *	Removed Empty (after trimmed) spaces.
 * @param fin File stream to read from
 * @param fields Container to put results
 * @param delim Delimiters to use
 * @return if line retrieval was successful
 */
bool fline_tr(ifstream* fin, vector<string>* fields, string delim)
{
  fields->clear();
  vector<string>::iterator p, q;
 
    string full;
    getline(*fin, full);
    
    if (fin->fail() && fin->eof())
      return false;
    
    boost::split(*fields, full, boost::is_any_of(delim.c_str()));
    
  for (p = fields->begin(); p != fields->end(); p++)
    {
      while (trim(*p).compare("") == 0 && p != fields->end())
      {
	q = p;
	fields->erase(q);
      }
      
      if (p == fields->end())
	break;
      
      *p = trim(*p);
    }
  
  return true;
}

/**
 * @fn map < string, vector < string > > getParameters(int argc, char** input)
 * 
 * 	Return a list of parameter names and values from a string. The names can be preceeded by 
 * nothing, a '-' or a '--'. The value then follows an '='. Values that may contain nested flags should
 * be surrounded by single quotes (').
 * 
 * @param argc number of elements in input
 * @param input array of strings to parse for flags and arguments
 * 
 * @return A map related each flag to its given string value.
 */
map < string, vector < string > > getParameters(int argc, char** input){
    vector<string> fields, subfields;
    vector<string>::iterator p, q;
    map < string, vector < string > > results;
    string argIn = "";
    
    for (sui t = 1; t < argc; t++){
      string flag = input[t];
      
	boost::split(fields, flag, boost::is_any_of("+"));
	string allArgs = "";
	
      if (fields.size() > 1){
	size_t pos;
	while ((pos = fields[0].find("-")) != string::npos)
	  fields[0].erase(pos, 1);
	
	while ((pos = fields[0].find("=")) != string::npos)
	  fields[0].erase(pos, 1);
	
	allArgs += fields[fields.size()-1];
	
	while (true){
	  if (t+1 >= argc){
	    cerr << "Something wrong with args input" << endl;
	    exit(3);
	  }
	  
	  string next = input[t+1];
	  
	  if (next.at(next.size() - 1) == '+')
	  {
	    allArgs+= " " + next.substr(0, next.size()-1);
	    vector <string> temp;
	    temp.push_back(allArgs);
	    results.insert(make_pair<string, vector <string> >(fields[0], temp));
	    t++;
	    break;
	  }else{
	    allArgs += " " + next;
	    t++;
	  }
	}
	  
      } else {
	argIn += boost::lexical_cast<string>(input[t]) + " "; 
      }
    }
    
    /*Split on "-" to find flags*/
      boost::split(fields, argIn, boost::is_any_of("-"));
  
    /*Get rid of empties*/
      for (p = fields.begin(); p != fields.end(); p++)
      {
	while (trim(*p).compare("") == 0 && p != fields.end())
	{
	  q = p;
	  fields.erase(q);
	}
	
	if (p == fields.end())
	  break;
	
	*p = trim(*p);
      }
      
      
      for (sui i = 0; i < fields.size(); i++){
	/*Split flag from value*/
	boost::split(subfields, fields[i], boost::is_any_of("= "));
	
	for (p = subfields.begin(); p != subfields.end(); p++)
	{
	  while (trim(*p).compare("") == 0 && p != subfields.end())
	  {
	    q = p;
	    subfields.erase(q);
	  }
	  
	  if (p == subfields.end())
	    break;
	  
	  *p = trim(*p);
	}
	
	vector <string> temporary;
	for (sui k = 1; k < subfields.size(); k++)
	  temporary.push_back(subfields[k]);
	
	results.insert(make_pair<string, vector <string> >(subfields[0], temporary));
      }
    
    return results;
}

/**
 * @fn int lineCount(string filename)
 *
 * Count the number of lines in a given file
 *
 * @param filename File to count lines of
 */
int lineCount(string filename){
  ifstream fin;
  openFile(&fin, filename.c_str());
  vector <string> fields;
  int count = 0;
  
  while (fline_tr(&fin, &fields, "\n"))
    count++;
  
  return count;
}

