/**
 * @file density.cc
 */

#include <boost/graph/adjacency_list.hpp>

#include "density.h"

/**
 * @fn void calculate_win_wout(const bidirected_t::graph& g, const std::set<bidirected_t::vertex>& s, bidirected_t::edge_name_t* win, bidirected_t::edge_name_t* wout)
 * @brief Calculates win and wout
 * 
 * Calculates the weights of all edges contained within set s (saved in win) and those edges
 * that split between a vertex in the set and a vertex elsewhere (saved in wout).
 *
 * @param g A full network
 * @param s a set of vertices that are to be taken as a 'cluster'
 * @param win a pointer to a lui that will be used to represent the weight of the inside edges
 * @param wout a pointer to a lui that will be used to represent the weight of the outside edges.
 */
void calculate_win_wout(const bidirected_t::graph& g,
        const std::set<bidirected_t::vertex>& s,
        bidirected_t::edge_name_t* win, bidirected_t::edge_name_t* wout)
{
    *win = 0.0;
    *wout = 0.0;

	/*Looping through all the vertices in the 'cluster'*/
    for (std::set<bidirected_t::vertex>::const_iterator i = s.begin();
            i != s.end(); ++i)
    {

        boost::graph_traits<bidirected_t::graph>::in_edge_iterator is, ie; // ?
        boost::graph_traits<bidirected_t::graph>::out_edge_iterator os, oe;

		/* out_edges gets a list of the outgoing edges in graph g connected to the vertex that the iterator
		 * i is connected to. Using tie() as such makes os the start of said list and oe the end of the list.
		 */
        for (boost::tie(os, oe) = boost::out_edges(*i, g); os != oe; ++os)
        {
			//Get the vertex that the edge was going to
            bidirected_t::vertex v = boost::target(*os, g);

		    /*
		      * If the target edge is also in the set of vertices given to the function adds 'edge_name' to win,
		      *  otherwise adds to wout
		      */
            if (s.find(v) == s.end())
            {
                *wout   += boost::get(boost::edge_name, g, *os);
            }
            else
            {
                *win    += boost::get(boost::edge_name, g, *os);
            }
        }
    }
}
