/**
 * @file density.h
 * @brief Holds definitions of different cluster density measures
 */

#ifndef DENSITY_H_
#define DENSITY_H_

#include "graph.h"

/**
 * @class density 
 * @brief Interface for densities 
 *
 * Provides a function to numerically suggest how dense a certain cluster is.
 */
template <typename graph_t>
class density
{
    public:
      /**
       * Destructor 
       * 	Empty
       */
        virtual ~density() {};
	
	/**
	 * Calculates a density based on the above numbers.
	 * 
	 * @param win Sum of weights of edges with both endpoints in cluster
	 * @param wout Sum of weights of edges with exactly one endpoint in cluster
	 * @param k Size of the cluster
	 * 
	 * @return Density value
	 */
        virtual double operator ()(typename graph_t::edge_name_t win, typename graph_t::edge_name_t wout, lui k) const =0;
};

/**
 * @class simple_density
 * @brief Ratio of internal weights to internal+external weights
 * 
 * Not really useful
 */
template <typename graph_t>
class simple_density: public density<graph_t>
{
    public:
      /**
	 * Calculates a density based on the above numbers.
	 * 
	 * @param win Sum of weights of edges with both endpoints in cluster
	 * @param wout Sum of weights of edges with exactly one endpoint in cluster
	 * @param k Size of the cluster
	 * 
	 * @return win / (win + wout + 1)
	 */
        double operator ()(typename graph_t::edge_name_t win,
            typename graph_t::edge_name_t wout, lui k) const
        {
            k = k;  // ?
            double num = win;
            double denom = win + wout + 1;
            return num / denom;
        }
};


/**
 * @class complex_density
 * @brief Weights simple_density measure with edge probability
 * 
 * Weighted equation of the simple definition and 
 *  a ratio of the weights of included edges to the number of possible inside edges.
 */
template <typename graph_t>
class complex_density: public density<graph_t>
{
    public:
      /**
       * Constructor
       * 
       * @param _lambda Parameter to control the focus balance between simple_density measure and edge probability (0 - 0.5 work best)
       */
        complex_density(double _lambda): lambda(_lambda) {};

	/**
	 * Calculates a density based on the above numbers.
	 * 
	 * @param win Sum of weights of edges with both endpoints in cluster
	 * @param wout Sum of weights of edges with exactly one endpoint in cluster
	 * @param k Size of the cluster
	 * 
	 * @return (1-lambda)*simple_density + lambda*edge probability
	 */
        double operator ()(typename graph_t::edge_name_t win,
            typename graph_t::edge_name_t wout, lui k) const
        {
            if (k <= 1 || (win == 0 && wout == 0))
            {
                return 0;
            }
            double part1 = (1 - lambda) * win/(win + wout);
            double part2 = lambda * win / (k * (k - 1));
            return part1 + part2;
        }

    private:
        double lambda; /**< @see Constructor*/
};

/**
 * @class negative_density
 * @brief For use in networks with negative edge weights
 * 
 * simple_density and complex_density will not work with negative edges. Negative_density will only work with negative edges. 
 * (Not entirely true, the communities are senseless if used in an all-positive network)
 */
template <typename graph_t>
class negative_density: public density<graph_t>
{
    public:
      /**
       * Constructor
       * 
       * @param param Lambda value to put weight on edge probability
       */
        negative_density(double param):lambda(param) {};
	
	/**
	 * Calculates a density based on the above numbers.
	 * 
	 * @param win Sum of weights of edges with both endpoints in cluster
	 * @param wout Sum of weights of edges with exactly one endpoint in cluster
	 * @param k Size of the cluster
	 * 
	 * @return (win-wout)/k
	 */
        double operator ()(typename graph_t::edge_name_t win,
            typename graph_t::edge_name_t wout, lui k) const
        {
	  if (k <= 0)
            {
                return -numeric_limits<double>::max();
            }
            double part1 = (1 - lambda) * (double)((win)-(wout))/(double)k;
            double part2;
	  if (k == 1)
	    part2 = 1;
	  else if (k > 1)
	    part2 = lambda * win / (k * (k - 1));
	  else
	    cerr << "BAD DENSITY ROBOT" << endl;
	    //Error
          return part1 + part2;
        }

    private:
        double lambda;
};

/**
 * @fn void calculate_win_wout(const bidirected_t::graph& g, const std::set<bidirected_t::vertex>& s, bidirected_t::edge_name_t* win, bidirected_t::edge_name_t* wout)
 * @brief Calculates win and wout
 * 
 * Calculates the weights of all edges contained within set s (saved in win) and those edges
 * that split between a vertex in the set and a vertex elsewhere (saved in wout).
 */
void calculate_win_wout(const bidirected_t::graph& g,
        const std::set<bidirected_t::vertex>& s,
        bidirected_t::edge_name_t* win, bidirected_t::edge_name_t* wout);


#endif
