/**
 * @file graph.h
 * @brief Boost network helper structures
 */

#ifndef GRAPH_H_
#define GRAPH_H_

#include "MyTools.h"
#include <iostream>
#include <map>
#include <vector>

#include <boost/graph/adjacency_list.hpp>

#include "types.h"

/** 
 * @struct undirected_t
 * @brief Undirected Networks
 * 
 * An undirected graph type suitable for connected_components
 */
struct undirected_t
{
    typedef boost::adjacency_list<boost::setS, boost::vecS, boost::undirectedS,
            boost::property<boost::vertex_name_t, double>,
            boost::property<boost::edge_name_t, double> > graph; /**< Network structure */
    typedef boost::graph_traits<graph>::vertex_descriptor vertex; /**< Network vertex or node */
    typedef boost::graph_traits<graph>::edge_descriptor edge; /**< Network edge */
    typedef boost::graph_traits<graph>::edge_iterator edge_iterator; /**< Iterator for network edges*/
    typedef std::map<lui, vertex> map; /**< ID or vertex name to vertex structure mapping */
    typedef double edge_name_t; /**< Edge identifier*/
    typedef lui vertex_name_t; /**< vertex identifier */
};


/**
 * @struct directed_t
 * @brief Directed Networks
 * 
 * A directed graph type suitable for adding and removing vertices many times.
 */
struct directed_t
{
    typedef boost::adjacency_list<boost::setS, boost::listS, boost::directedS,
            boost::property<boost::vertex_name_t, lui>,
            boost::property<boost::edge_name_t, lui> > graph; /**< Network structure */
    typedef boost::graph_traits<graph>::vertex_descriptor vertex; /**< Network vertex or node */
    typedef boost::graph_traits<graph>::edge_descriptor edge; /**< Network edge */
    typedef boost::graph_traits<graph>::edge_iterator edge_iterator;/**< Iterator for network edges*/
    typedef std::map<lui, vertex> map;/**< ID or vertex name to vertex structure mapping */
    typedef double edge_name_t; /**< Edge identifier*/
    typedef lui vertex_name_t; /**< vertex identifier */
};


/**
 * @struct bidirected_t
 * @brief Directed Network With Cycles of Size 2
 * 
 * A directed graph type suitable for adding and removing vertices many times.
 */
struct bidirected_t
{
    typedef boost::adjacency_list<boost::setS, boost::listS, boost::bidirectionalS,
            boost::property<boost::vertex_name_t, lui>,
            boost::property<boost::edge_name_t, double> > graph; /**< Network structure */
    typedef boost::graph_traits<graph>::vertex_descriptor vertex; /**< Network vertex or node */
    typedef boost::graph_traits<graph>::edge_descriptor edge; /**< Network edge */
    typedef boost::graph_traits<graph>::edge_iterator edge_iterator; /**< Iterator for network edges*/
    typedef std::map<lui, vertex> map;/**< ID or vertex name to vertex structure mapping */
    typedef double edge_name_t; /**< Edge identifier*/
    typedef lui vertex_name_t; /**< vertex identifier */
};


/**
 * @fn typename graph_t::vertex add_or_get_vertex(typename graph_t::graph &g, typename graph_t::map &vertices, lui id)
 * @brief Retrieves a vertex from the network
 * 
 * Converts from regular vertex ID to boost vertex structure. Adds the vertex into the network if it's not already there.
 * 
 * @param g The network
 * @param vertices The vertex map
 * @param id Vertex to retrieve
 * 
 * @return Boost vertex corresponding to given vertex ID
 */
template <typename graph_t>
typename graph_t::vertex add_or_get_vertex(typename graph_t::graph &g,
        typename graph_t::map &vertices, lui id)
{
    typename graph_t::map::iterator pos;
    bool inserted;
    typename graph_t::vertex v;

    boost::tie(pos, inserted) = vertices.insert(
            std::make_pair(id, typename graph_t::vertex()));

    if (inserted)
    {
        v = add_vertex(g);
        boost::put(boost::vertex_name, g, v, id);
        pos->second = v;
    }
    else
    {
        v = pos->second;
    }
    return v;
}


/**
 * @fn void bump_edge(typename graph_t::graph &g, typename graph_t::map &vertices, typename graph_t::vertex_name_t fr, typename graph_t::vertex_name_t to)
 * 
 * A function to bump the weight of an edge or create and set it to 1.
 * 
 * @param g The network
 * @param vertices Boost vertex map for the network
 * @param fr An endpoint of the edge
 * @param to The other endpoint
 */
template <typename graph_t>
void bump_edge(typename graph_t::graph &g, typename graph_t::map &vertices,
        typename graph_t::vertex_name_t fr, typename graph_t::vertex_name_t to)
{
    typename graph_t::vertex u, v;
    typename graph_t::edge_name_t weight;
    u = add_or_get_vertex<graph_t>(g, vertices, fr);
    v = add_or_get_vertex<graph_t>(g, vertices, to);

    typename graph_t::edge e;
    bool inserted;
    boost::tie(e, inserted) = boost::add_edge(u, v, g);
    if (inserted)
    {
        boost::put(boost::edge_name, g, e, 1);
    }
    else
    {
        weight = boost::get(boost::edge_name, g, e);
        boost::put(boost::edge_name, g, e, weight + 1);
    }
}

/**
 * @fn void read_from_stream(typename graph_t::graph& g, typename graph_t::map& vertices, map < string, lui >& id, std::ifstream& in, double alpha = 1, double beta = 1, double weightThresh = 0, string delimiter = "\t,")
 * 
 * Add edges to the graph according the the (fr, to, wt) triples in the file.
 * 
 * @param g The network
 * @param vertices The boost vertex map
 * @param id Original vertex name to ID map
 * @param in Stream to read network from
 * @param alpha factor for positive edges
 * @param beta factor for negative edges
 * @param weightThresh Minimum Edge Weight
 * @param delimiter Network file delimiters
 */
template <typename graph_t>
void read_from_stream(typename graph_t::graph& g, typename graph_t::map& vertices, map <string, lui>& id,
        std::ifstream& in, double alpha, double beta, double weightThresh, string delimiter)
{
    typename graph_t::vertex_name_t fr;
    typename graph_t::vertex_name_t to;
    typename graph_t::edge_name_t wt;

    vector <string> fields;
    map <string, lui>::iterator mit;
    lui NextId = 0;
    
    for (mit = id.begin(); mit != id.end(); mit++)
    {
      if (mit->second > NextId)
	NextId = mit->second;
    }
    
    NextId++;

    /**
     * Get the maximum weight absolute value to normalize all edge weights 
     * (helps normalize the second part of complex density calculation)
     */
    double maxWeight = 0;

    while (fline_tr(&in, &fields, delimiter.c_str()))
      {
	if (fields.size() != 3)
	  continue;

	double dw = boost::lexical_cast<double>(fields[2]);
	if (abs(dw) > maxWeight)
	  maxWeight = abs(dw);
      }

    beta = beta / maxWeight;
    alpha = alpha / maxWeight;

    in.clear();
    in.seekg(0, ios::beg);

    ofstream gout("TrueNetwork.dat");
    while (fline_tr(&in, &fields, delimiter.c_str()) )
    {
      if (fields.size() != 3)
	continue;

      double dw = boost::lexical_cast<double>(fields[2]);
      if (dw < 0)
	dw *= beta;
      else
	dw *= alpha;
      
      if (abs(dw) < weightThresh)
	continue;
      
      wt = boost::lexical_cast<typename graph_t::edge_name_t>(dw);
      
      if ((mit = id.find(fields[0])) != id.end())
	fr = mit->second;
      else
      {
	id.insert(make_pair<string, int>(fields[0], NextId));
	fr = NextId++;
      }
      
      if ((mit = id.find(fields[1])) != id.end())
	to = mit->second;
      else
      {
	id.insert(make_pair<string, int>(fields[1], NextId));
	to = NextId++;
      }
      
      gout << fr << ", " << to << ", " << dw << endl;
      
      typename graph_t::vertex u, v;
      u = add_or_get_vertex<graph_t>(g, vertices, fr);
      v = add_or_get_vertex<graph_t>(g, vertices, to);

      if (u == v)
      {
	  // Disallow self-edges.
	  continue;
      }

      typename graph_t::edge e;
      bool inserted;
      boost::tie(e, inserted) = boost::add_edge(u, v, g);
      if (inserted)
      {
	  boost::put(boost::edge_name, g, e, wt);
      }
      else
      {
	typename graph_t::vertex_name_t a, b;
	a = boost::get(boost::vertex_name, g, u);
	b = boost::get(boost::vertex_name, g, v);
	cerr << a << " " << b << endl;
	  throw "Duplicate edge not allowed. ";
      }
    }
}

/**
 * @fn void print_set(const typename graph_t::graph& g, const std::set<typename graph_t::vertex>& v, std::ostream& out, float density)
 * 
 * Print a line for the set of vertices.  Use the vertex_name as labels.
 * 
 * @param g The network
 * @param v Set of vertices to print
 * @param out File to print to
 * @param density Density of the set of vertices
 */
template <typename graph_t>
void print_set(const typename graph_t::graph& g,
	       const std::set<typename graph_t::vertex>& v, std::ostream& out, float density)
{
  if (density <= 0)
    return;
  
  string outString = "";
  if (v.size() > 2)
  {
    outString += boost::lexical_cast<string>(density) + " " + boost::lexical_cast<string>(v.size()) + " ";
    typename std::set<typename graph_t::vertex>::const_iterator i;
    for (i = v.begin(); i != v.end(); ++i)
    {
        typename graph_t::vertex_name_t u;
        u = boost::get(boost::vertex_name, g, *i);
        outString += boost::lexical_cast<string>(u) + " ";
    }
    outString += "\n";
    out << outString;
    out.flush();
  }
}
/**
 * @fn void print_set_trans(const typename graph_t::graph& g,
        const std::set<typename graph_t::vertex>& v, std::ostream& out, float density, map<string, lui> ID, char delim)
 * 
 * Print a line for the set of vertices using the original vertex identifiers
 * 
 * @param g The network
 * @param v Set of vertices to print
 * @param out File to print to
 * @param density Density of the set of vertices
 * @param ID VertexID to original name mapping
 * @param delim Character to put between vertex names in results file.
 */
template <typename graph_t>
void print_set_trans(const typename graph_t::graph& g,
		     const std::set<typename graph_t::vertex>& v, std::ostream& out, float density, map<string, lui> ID, char delim)
{
  if (density <= 0){
    return;
  }
  map <lui, string> rID;
    map <lui, string>::iterator it_r;
    map <string, lui>::iterator it_id;
    set <lui>::iterator it_sl; 
    
    for (it_id = ID.begin(); it_id != ID.end(); it_id++){
      rID.insert(make_pair<lui, string>(it_id->second, it_id->first));
    }
    
  string outString = "";
  if (v.size() > 2)
  {
    outString += boost::lexical_cast<string>(density) + boost::lexical_cast<string>(delim) + " " + boost::lexical_cast<string>(v.size()) + boost::lexical_cast<string>(delim) + " ";
    typename std::set<typename graph_t::vertex>::const_iterator i;
    for (i = v.begin(); i != v.end(); ++i)
    {
        typename graph_t::vertex_name_t u;
        u = boost::get(boost::vertex_name, g, *i);
        outString += (rID.find(boost::lexical_cast<lui>(u)))->second + boost::lexical_cast<string>(delim) + " ";
    }
    outString = outString.substr(0, outString.size()-2) + "\n";
    out << outString;
    out.flush();
  }
}
#endif
