/**
 * @file iterativescan.cc
 * @brief Main CIS implementation
 */

#include "iterativescan.h"
#include <iostream>

/**
 * @fn std::pair<std::vector<seed>*, double> expand_seed(seed S)
 * 
 * The pair returned is a vector of all disjoint subgraphs within a modified seed S (one step of addition
 * and removal) and a number representing how closely connected the modified seed is. 
 *
 * NOTE: seed S doesn't actually change as it is passed by value.
 * 
 * @param S Set of vertices to expand
 * 
 * @return Results - could be multiple because of fragmentation
 */

std::pair<std::vector<seed>*, double> expand_seed(seed S)
{
    const bidirected_t::graph& g(S.graph());
    double old_density = S.dens();

    // Collect the fringe of the seed.
    typedef std::pair<lui, bidirected_t::vertex> degree_vertex_pair;
    std::set<degree_vertex_pair> peers;

	/*Go through all the vertices in the seed*/
    std::set<bidirected_t::vertex>::const_iterator si;
    for (si = S.vertices().begin(); si != S.vertices().end(); ++si)
    {
        boost::graph_traits<bidirected_t::graph>::in_edge_iterator is, ie;
        boost::graph_traits<bidirected_t::graph>::out_edge_iterator os, oe;

		/*Insert the current vertex into the list, along with its degree*/
        bidirected_t::vertex u = *si;
        lui degree = boost::degree(u, g);
        peers.insert(std::make_pair(degree, u));

		/*Go through all the vertices connected to the current vertex and add them to the 
		 * list as well (along with their vertices).
		 */
        for (boost::tie(os, oe) = boost::out_edges(*si, g); os != oe; ++os)
        {
            bidirected_t::vertex v = boost::target(*os, g);
            lui degree = boost::degree(v, g);
            peers.insert(std::make_pair(degree, v));
        }
        for (boost::tie(is, ie) = boost::in_edges(*si, g); is != ie; ++is)
        {
            bidirected_t::vertex v = boost::source(*is, g);
            lui degree = boost::degree(v, g);
            peers.insert(std::make_pair(degree, v));
        }
    }

    // The fringe is now sorted by increasing degree by the set.

    // For each vertex in the set of peers, test if the addition or removal of
    // the vertex impacts the set's density.

	/*NOTE: this should probably be done multiple times, whether that happens here or through
	 * multiple calls to the function. Each function call will change the outlook of the set, 
	 * and may change which vertices can be added/removed.
	 */
    std::set<degree_vertex_pair>::const_iterator pi;
    for (pi = peers.begin(); pi != peers.end(); ++pi)
    {   
        if (S.can_add_vertex(pi->second))
        {
            S.add_vertex(pi->second);
	   // typename bidirected_t::vertex_name_t u;
	    //u = boost::get(boost::vertex_name, g, pi->second);
	    //cerr << "Added " << u << " : " << S.dens() << "  { " << S.win() << " " << S.wout() << " " << S.vertices().size() << " } " << endl;
        }
        else if (S.can_rm_vertex(pi->second))
        {
            S.rm_vertex(pi->second);
	   // typename bidirected_t::vertex_name_t u;
	    //u = boost::get(boost::vertex_name, g, pi->second);
	    //cerr << "Removed " << u << " : " << S.dens() << "  { " << S.win() << " " << S.wout() << " " << S.vertices().size() << " } " << endl;
        }
    }

    double new_density = S.dens();
    
    //print_set<bidirected_t>(S.graph(), S.vertices(), cout, S.dens());

    std::vector<seed>* pieces = fracture_seed(S);

    double difference;

    if (new_density == 0)
    {
        // new_density can only be zero if there was no means for the cluster to
        // expand (cluster of one).  Consider that each seed starts with a density of zero.  As
        // nodes are added/removed, the density improves.  Therefore, the only
        // way the density can remain 0 is for no nodes to be added or removed.
        //
        // In any case, we don't want a floating point exception should we
        // encounter a single-vertex component.
        difference = 0;
    }
    else
    {
        difference = (new_density - old_density) / new_density;
	//cerr << new_density << " " << old_density << endl;
    }

    return std::make_pair(pieces, difference);
}


