/**
 * @file seed.h
 * @brief Declarations for seed community in CIS
 */ 
#ifndef SEED_H_
#define SEED_H_

#include <set>
#include <iostream>
#include <vector>

#include "density.h"
#include "graph.h"

/**
 * @class seed
 * @brief Potential Cluster
 * 
 * Keeps track of internal density, external density, vertices in seed, a density class, and provides functions to help decided if vertices
 * should be added or removed.
 */
class seed
{
    public:
        typedef std::set<directed_t::vertex> vertex_set; /**< Set of vertices (seed or community)*/

	/**
	 * Constructor
	 * 	
	 * @param dens Density class to compute 'fitness' function
	 * @param g The network
	 */
        seed(const density<bidirected_t>* dens, const bidirected_t::graph& g);
	/**
	 * Constructor
	 * 	
	 * @param dens Density class to compute 'fitness' function
	 * @param g The network
	 * @param s Vertex to initialize seed with
	 */
        seed(const density<bidirected_t>* dens, const bidirected_t::graph& g,
                bidirected_t::vertex s);
	/**
	 * Copy Constructor
	 * 
	 * @param other Seed to copy
	 */
        seed(const seed& other);
	/**
	 * Operator overload for assignment operator
	 * 
	 * @param rhs Seed to set equal to
	 */
        seed& operator =(const seed& rhs);

	/**
	 * Checks to see if adding a vertex will increase the density value
	 * 
	 * @param v Vertex to consider adding
	 */
        bool can_add_vertex(const bidirected_t::vertex& v) const;
	/**
	 * Checks to see if removing a vertex will increase the density value
	 * 
	 * @param v Vertex to consider removing
	 */
        bool can_rm_vertex(const bidirected_t::vertex& v) const;
	/**
	 * Adds a vertex to the seed
	 * 
	 * @param v Vertex to add
	 */
        void add_vertex(const bidirected_t::vertex& v);
	/**
	 * Removes a vertex to the seed
	 * 
	 * @param v Vertex to remove
	 */
        void rm_vertex(const bidirected_t::vertex& v);

	/**
	 * Get Function
	 * 
	 * @return Sum of internal edge weights
	 */
        const bidirected_t::edge_name_t& win() const { return win_; };
	/**
	 * Get Function
	 * 
	 * @return Sum of external edge weights
	 */
        const bidirected_t::edge_name_t& wout() const { return wout_; };
	/**
	 * Get Function
	 * 
	 * @return The network
	 */
        const bidirected_t::graph& graph() const { return *g_; };
	/**
	 * Get Function
	 * 
	 * @return the density class
	 */
        const density<bidirected_t>* dens_func() const { return dens_; };
	/**
	 * Get Function
	 * 
	 * @return Set of vertices in seed
	 */
        const vertex_set& vertices() const { return vertices_; };
	/**
	 * Calculate the density
	 * 
	 * @return Density of seed
	 */
        double dens() const { return (*dens_)(win_, wout_, vertices_.size()); };
	
	/**
	 *  Print the seed
	 */
	void print();

    private:
      /**
       * Calculate the contribution of vertex p to the internal and external densities of the seed
       * 
       * @param v Vertex to consider
       * @param p_plus Edge weights connected inside seed
       * @param p_minus Edge weight connected outside seed
       */
        void find_p(const bidirected_t::vertex& v,
                bidirected_t::edge_name_t* p_plus,
                bidirected_t::edge_name_t* p_minus,
                bidirected_t::edge_name_t* p_not) const;

        const bidirected_t::graph* g_; /**< The network */
        const density<bidirected_t>* dens_; /**< The density calculation */
        bidirected_t::edge_name_t win_; /**< The sum of internal weights*/
        bidirected_t::edge_name_t wout_; /**< The sum of external weights*/
        vertex_set vertices_; /**< Vertices in the cluster*/
};

/**
 * Break a seed into its connected components.
 * 
 * @param S Seed to break apart
 * @return List of connected components
 */
std::vector<seed>* fracture_seed(const seed& S);

/**
 * Recursive call to break into connected components
 * 
 */
void fracture_seed(const seed& S, seed::vertex_set& seen,
        seed& P, bidirected_t::vertex v);

#endif
