///  This Source Code Form is subject to the terms of the Mozilla Public
///  License, v. 2.0. If a copy of the MPL was not distributed with this
///  file, You can obtain one at http://mozilla.org/MPL/2.0/.

#include "significant_frequency.cpp"
#include <iostream>
#include "tree_mining.cpp"

void Read_Data(ifstream& in_str, vector<communication>& comms){
    communication c;
    string s;
    while(!in_str.eof()){
        in_str >> s;
        if(in_str.eof()){break;}
        c.speaker = atoi(s.c_str());
        in_str >> s;
        c.listener = atoi(s.c_str());
        in_str >> s;
        c.time = atof(s.c_str());
        comms.push_back(c);
    }
}


void Handle_Data(vector<communication>& communications, ofstream& out_str, double minimum, double maximum, double delta, int num_repeats, int std_dev_offset, string triple_type, string cliques_or_freq, int significant_frequency = 1){
    vector<int> std_devs;
    vector<double> synthetic_data;

   //Gets Frequency list and standard-deviation list for synthetic data
    if(cliques_or_freq != "cliques"){
        Get_synthetic_data(communications, synthetic_data, std_devs, minimum, maximum, delta, num_repeats);
    }

    vector<int> actual_frequencies;
    int remaining_total;
    triple_match_map* tmap;

    //finds actual_frequencies if the cliques must also be calculated
    if(cliques_or_freq != "frequency"){
        tmap = Significant_Matches(communications, minimum, maximum, delta, significant_frequency, triple_type);
        remaining_total = tmap -> size();

        triple_match_map::iterator map_itr = tmap -> begin();
        while(map_itr != tmap -> end()){
            int current_frequency = (map_itr -> second) -> size();
            while(actual_frequencies.size() <= current_frequency){actual_frequencies.push_back(0);}
            actual_frequencies[current_frequency]++;
            map_itr++;
        }
    }
    else{ //finds actual_frequencues if the cliques do not need to be calculated
        map<triple, int>* triple_frequencies = Significant_Frequencies(communications, minimum, maximum, delta, 1, triple_type);
        remaining_total = triple_frequencies -> size();
        //counts all frequencies in list
        map<triple, int>::iterator freq_itr = triple_frequencies -> begin();
        while(freq_itr != triple_frequencies -> end()){
            while(actual_frequencies.size() <= freq_itr -> second){actual_frequencies.push_back(0);}
            actual_frequencies[freq_itr -> second]++;
            freq_itr++;
        }
    }

    //makes it so actual_frequencies[i] stores the number of triples with frequency >= i
    for(int j=1; j<actual_frequencies.size(); j++){
        int t = actual_frequencies[j];
        actual_frequencies[j] = remaining_total;
        remaining_total -= t;
    }

    //Calculates the frequency at which the actual data can be seen as significant
    if(cliques_or_freq != "cliques"){
        significant_frequency = get_Frequency(actual_frequencies, synthetic_data, std_devs, std_dev_offset);
    }

    //Finds the cliques and deletes tmap
    if(cliques_or_freq != "frequency"){
        //removes triples that occur less than the significant frequency
        for(triple_match_map::iterator t_map_itr = tmap -> begin(); t_map_itr != tmap -> end();){
            triple_match_map::iterator temp_itr = t_map_itr;
            t_map_itr++;
            if(temp_itr -> second -> size() < significant_frequency){tmap -> erase(temp_itr);}
        }
        
        //creates complete graph and finds cliques on this graph
        set<graph_edge>* g = Weighted_Graph(tmap);
        Write_Graph_To_File(g);
        system("./CIS_files/CIS -alg=CIS -i=temporary.txt -o=clustered_graph.txt -args=0 -tr");

        //Deletes tmap
        for(triple_match_map::iterator t_map_itr = tmap -> begin(); t_map_itr != tmap -> end(); t_map_itr++){
            delete t_map_itr -> second;
        }
        delete tmap;
    }

   //Outputs results to given file
    out_str << "min, max, delta: " << minimum << ", " << maximum << ", " << delta << endl;
    out_str << "communications: " << communications.size() << endl;
    out_str << "significant frequency: " << significant_frequency << endl;
    if(cliques_or_freq != "cliques"){out_str << "synthetic data repeats: " << num_repeats << endl;}
    out_str << "Frequency actual_data";
    if(cliques_or_freq != "cliques"){out_str << " synthetic_data std_dev";}
    out_str << endl;
    int i = 1;
    while(i < synthetic_data.size() || i < actual_frequencies.size()){
        int a_freq; int s_freq; int std_dev;
        if (i >= actual_frequencies.size()){a_freq = 0;}
        else{a_freq = actual_frequencies[i];}
        if(i >= synthetic_data.size()){s_freq = 0; std_dev = 0;}
        else{s_freq = synthetic_data[i]; std_dev = std_devs[i];}
        out_str << i << " " << a_freq;
        if(cliques_or_freq != "cliques") {out_str << " " << s_freq << " " << std_dev;}
        out_str << endl;
        i++;
    }
}

int main(int argc, char* argv[]){
   //Reads in arguments
    if(argc>9 || argc<5){cerr << "Arguments should be: input_file triple_type [minimum maximum delta] cliques_or_freq [synthetic_repeats std_dev_factor user_significance_threshold] \n"; return 0;}
    ifstream in_str(argv[1]);
    ofstream out_str("handle_data_output.txt");
    string triple_type = argv[2];
    int current_input = 3;
    double minimum = 0; double maximum = 0; double delta = 0;
    if(triple_type != "sibling"){ //if triple_type = "chain" or "both"
        minimum = atof(argv[current_input]);
        current_input++;
        maximum = atof(argv[current_input]);
        current_input++;
    }
    if(triple_type != "chain"){ // triple_type = "sibling" or "both"
        delta = atof(argv[current_input]);
        current_input++;
    }
    string cliques_or_freq = argv[current_input];
    current_input++;
    int synthetic_repeats = 1; int std_dev_factor = 0; int user_significance_threshold = 1;
    if(cliques_or_freq != "cliques"){ // cliques_or_freq = "frequency" or "both"
        synthetic_repeats = atoi(argv[current_input]);
        current_input++;
        std_dev_factor = atoi(argv[current_input]);
        current_input++;
    }
    else{
        user_significance_threshold = atoi(argv[current_input]);
    }

   //Reads in data
   vector<communication> comms;
   Read_Data(in_str, comms);

   //Handles data and outputs to given file
   Handle_Data(comms, out_str, minimum, maximum, delta, synthetic_repeats, std_dev_factor, triple_type, cliques_or_freq, user_significance_threshold);
}
